@@ -533,7 +533,9 @@ void CppDumper::writeImplStart()
     cpp << l("#include \"") << headerName << l("\"") << Qt::endl;
     cpp << Qt::endl
         << QStringLiteral("#include <qscxmlinvokableservice.h>") << Qt::endl
-        << QStringLiteral("#include <qscxmltabledata.h>") << Qt::endl;
+        << QStringLiteral("#include <qscxmltabledata.h>") << Qt::endl
+        << QStringLiteral("#include <QtCore/qtmochelpers.h>") << Qt::endl;
+
     for (const QString &inc : std::as_const(includes)) {
         cpp << l("#include <") << inc << l(">") << Qt::endl;
     }
@@ -559,10 +561,10 @@ void CppDumper::writeImplBody(const GeneratedTableData &table,
         dataModelInitialization = l("stateMachine.setDataModel(&dataModel);");
         break;
     case DocumentModel::Scxml::JSDataModel:
-        dataModelField = l("QScxmlDataModel *dataModel;");
+        dataModelField = l("std::unique_ptr<QScxmlDataModel> dataModel;");
         dataModelInitialization = l(
-        "   dataModel = QScxmlDataModel::createScxmlDataModel(QStringLiteral(\"ecmascriptdatamodel\"));\n"
-        "   stateMachine.setDataModel(dataModel);\n"
+        "   dataModel.reset(QScxmlDataModel::createScxmlDataModel(QStringLiteral(\"ecmascriptdatamodel\")));\n"
+        "   stateMachine.setDataModel(dataModel.get());\n"
         );
         break;
     case DocumentModel::Scxml::CppDataModel:
